# coding=utf-8
"""
"""
from __future__ import absolute_import, unicode_literals

import bleach
from wtforms import TextAreaField

from abilian.i18n import _l
from abilian.web.forms import Form
from abilian.web.forms.fields import DateTimeField
from abilian.web.forms.filters import strip
from abilian.web.forms.validators import required
from abilian.web.forms.widgets import RichTextWidget

ALLOWED_TAGS = [
    'a',
    'abbr',
    'acronym',
    'b',
    'blockquote',
    'br',
    'code',
    'em',
    'i',
    'li',
    'ol',
    'strong',
    'ul',
    'h1',
    'h2',
    'h3',
    'h4',
    'h5',
    'h6',
    'p',
    'u',
    'img',
]

ALLOWED_ATTRIBUTES = {
    '*': ['title'],
    'p': ['style'],
    'a': ['href', 'title'],
    'abbr': ['title'],
    'acronym': ['title'],
    'img': ['src', 'alt', 'title'],
}

ALLOWED_STYLES = ['text-align',]

WIDGET_ALLOWED = {}
for attr in ALLOWED_TAGS:
    allowed = ALLOWED_ATTRIBUTES.get(attr, True)
    if not isinstance(allowed, bool):
        allowed = {tag: True for tag in allowed}
    WIDGET_ALLOWED[attr] = allowed


class EventForm(Form):
    start = DateTimeField(_l("Start"), validators=[required()])
    end = DateTimeField(_l("End"))

    location = TextAreaField(label=_l("Location"),
                             filters=(strip,),
                             validators=[required()])

    description = TextAreaField(
        label=_l("Description"),
        widget=RichTextWidget(allowed_tags=WIDGET_ALLOWED),
        filters=(strip,),
        validators=[required()])

    def validate_description(self, field):
        field.data = bleach.clean(field.data,
                                  tags=ALLOWED_TAGS,
                                  attributes=ALLOWED_ATTRIBUTES,
                                  styles=ALLOWED_STYLES,
                                  strip=True)



# coding=utf-8
"""
"""
from __future__ import absolute_import

import imghdr
from string import strip

import PIL
import sqlalchemy as sa
from flask import request
from flask_babel import gettext as _
from flask_babel import lazy_gettext as _l
from wtforms.fields import BooleanField, StringField, TextAreaField
from wtforms.validators import ValidationError, optional, required

from abilian.core.models.subjects import Group
from abilian.web.forms import Form
from abilian.web.forms.fields import FileField, Select2Field
from abilian.web.forms.validators import length
from abilian.web.forms.widgets import BooleanWidget, ImageInput, TextArea

from .models import Community


def _group_choices():
    m_prop = Group.members.property
    membership = m_prop.secondary
    query = Group.query.session.query(
        Group.id, Group.name, Community.name.label('community'),
        sa.sql.func.count(membership.c.user_id).label('members_count'))
    query = query.outerjoin(m_prop.secondary, m_prop.primaryjoin) \
        .outerjoin(Community, Community.group.property.primaryjoin) \
        .group_by(Group.id, Group.name, Community.name) \
        .order_by(sa.sql.func.lower(Group.name)) \
        .autoflush(False)
    choices = [(u'', u'')]

    for g in query:
        label = u'{} ({:d} membres)'.format(g.name, g.members_count)
        if g.community:
            label += u' — Communauté: {}'.format(g.community)
        choices.append((unicode(g.id), label))

    return choices


class CommunityForm(Form):
    name = StringField(label=_l(u"Name"), validators=[required()])
    description = TextAreaField(label=_l(u"Description"),
                                validators=[required(), length(max=500)],
                                widget=TextArea(resizeable="vertical"))

    linked_group = Select2Field(
        label=_l(u'Linked to group'),
        description=_l(
            u'Manages a group of users through this community members.'),
        choices=_group_choices)

    image = FileField(label=_l('Image'),
                      widget=ImageInput(width=65, height=65),
                      validators=[optional()])

    type = Select2Field(label=_(u"Type"), validators=[required()],
                        filters=(strip,),
                        choices=[(_l(u'informative'), 'informative'),
                                 (_l(u'participative'), 'participative')])

    has_documents = BooleanField(label=_l(u"Has documents"),
                                 widget=BooleanWidget(on_off_mode=True))
    has_wiki = BooleanField(label=_l(u"Has a wiki"),
                            widget=BooleanWidget(on_off_mode=True))
    has_forum = BooleanField(label=_l(u"Has a forum"),
                             widget=BooleanWidget(on_off_mode=True))

    def validate_name(self, field):
        name = field.data = field.data.strip()

        if name and field.object_data:
            # form is bound to an existing object, name is not empty
            if name != field.object_data:
                # name changed: check for duplicates
                if Community.query.filter(Community.name == name).count() > 0:
                    raise ValidationError(_(u"A community with this name already exists"))

    def validate_description(self, field):
        field.data = field.data.strip()

    # FIXME: code duplicated from the user edit form (UserProfileForm).
    # Needs to be refactored.
    def validate_image(self, field):
        data = request.form.get('image')
        if not data:
            return

        data = field.data
        filename = data.filename
        valid = any(map(filename.lower().endswith, ('.png', '.jpg', '.jpeg')))

        if not valid:
            raise ValidationError(_(u'Only PNG or JPG image files are accepted'))

        img_type = imghdr.what('ignored', data.read())

        if img_type not in ('png', 'jpeg'):
            raise ValidationError(_(u'Only PNG or JPG image files are accepted'))

        data.seek(0)
        try:
            # check this is actually an image file
            im = PIL.Image.open(data)
            im.load()
        except:
            raise ValidationError(_(u'Could not decode image file'))

        data.seek(0)
        field.data = data



from abilian.core.models.subjects import User
from abilian.sbe.apps.communities.models import READER, Community
from abilian.sbe.testing import BaseTestCase


class CommunityBaseTestCase(BaseTestCase):

    no_login = True

    def setUp(self):
        super(CommunityBaseTestCase, self).setUp()
        self.community = Community(name=u"My Community")
        self.session.add(self.community)
        self.session.flush()


class CommunityIndexingTestCase(CommunityBaseTestCase):
    """
    Testcase for testing indexing and searching of community content.
    """

    no_login = False
    SERVICES = ('security', 'indexing',)

    def setUp(self):
        super(CommunityIndexingTestCase, self).setUp()
        self.svc = self.app.services['indexing']
        self.user = User(email=u'user_1@example.com',
                         password='azerty',
                         can_login=True)
        self.session.add(self.user)
        self.community.set_membership(self.user, READER)
        self.c2 = Community(name=u'Other community')
        self.session.add(self.c2)
        self.user_c2 = User(email=u'user_2@example.com',
                            password='azerty',
                            can_login=True)
        self.session.add(self.user_c2)
        self.c2.set_membership(self.user_c2, READER)

        self.user_no_community = User(email=u'no_community@example.com',
                                      password='azerty',
                                      can_login=True)
        self.session.add(self.user_no_community)
        self.session.commit()



# coding=utf-8
"""
"""
from __future__ import absolute_import

from datetime import datetime, timedelta
from email.parser import FeedParser
from unittest import TestCase

import pytest
from flask import url_for
from mock import Mock, patch
from pathlib import Path

from abilian.core.models.subjects import User
from abilian.sbe.apps.communities.models import MANAGER, MEMBER
from abilian.sbe.apps.communities.tests.base import (CommunityBaseTestCase,
                                                     CommunityIndexingTestCase)
from abilian.sbe.testing import BaseTestCase

from ..commands import inject_email
from ..models import Post, Thread, ThreadClosedError
from ..tasks import (build_reply_email_address, extract_email_destination,
                     process)


class Test(TestCase):

    def test_create_post(self):
        thread = Thread(title="Test thread")
        post = thread.create_post()
        assert post in thread.posts
        assert post.name == u'Test thread'

        thread.title = u'new title'
        assert thread.name == u'new title'
        assert post.name == u'new title'

    def test_closed_property(self):
        thread = Thread(title=u'Test Thread')
        assert thread.closed is False
        thread.closed = True
        assert thread.closed is True
        thread.closed = 0
        assert thread.closed is False
        thread.closed = 1
        assert thread.closed is True
        assert thread.meta['abilian.sbe.forum']['closed'] is True

    def test_thread_closed_guard(self):
        thread = Thread(title=u'Test Thread')
        thread.create_post()
        thread.closed = True

        with pytest.raises(ThreadClosedError):
            thread.create_post()

        p = Post(body_html=u'ok')

        with pytest.raises(ThreadClosedError):
            p.thread = thread

        thread.closed = False
        p.thread = thread
        assert len(thread.posts) == 2

        thread.closed = True
        with pytest.raises(ThreadClosedError):
            del thread.posts[0]

        assert len(thread.posts) == 2

        with pytest.raises(ThreadClosedError):
            # actually thread.posts will be replaced by `[]` and we can't prevent
            # this, but exception has been raised
            thread.posts = []

        thread.closed = False
        thread.posts = [p]
        thread.closed = True
        assert thread.posts == [p]
        assert p.thread is thread
        with pytest.raises(ThreadClosedError):
            p.thread = None

    def test_change_thread_copy_name(self):
        thread = Thread(title=u'thread 1')
        thread2 = Thread(title=u'thread 2')
        post = Post(thread=thread, body_html=u'post content')
        assert post.name == thread.name

        post.thread = thread2
        assert post.name == thread2.name

    def test_task_process_email(self):
        """
        test the process_email function
        """
        marker = u'_____Write above this line to post_____'

        message = get_emailmessage_from_file('reply.email')
        newpost = process(message, marker)
        assert newpost

        message = get_emailmessage_from_file('reply_nocharset_specified.email')
        newpost = process(message, marker)
        assert newpost

        message = get_emailmessage_from_file('reply_no_marker.email')
        self.assertRaises(LookupError, process, message, marker)

        # dubious check
        message = get_emailmessage_from_file('reply_no_textpart.email')
        self.assertRaises(LookupError, process, message, marker)


class IndexingTestCase(CommunityIndexingTestCase):

    def test_thread_indexed(self):
        thread = Thread(title=u'Community 1', community=self.community)
        self.session.add(thread)
        thread_other = Thread(title=u'Community 2: other', community=self.c2)
        self.session.add(thread_other)
        self.session.commit()

        svc = self.svc
        obj_types = (Thread.entity_type,)
        with self.login(self.user_no_community):
            res = svc.search(u'community', object_types=obj_types)
            assert len(res) == 0

        with self.login(self.user):
            res = svc.search(u'community', object_types=obj_types)
            assert len(res) == 1
            hit = res[0]
            assert hit['object_key'] == thread.object_key

        with self.login(self.user_c2):
            res = svc.search(u'community', object_types=obj_types)
            assert len(res) == 1
            hit = res[0]
            assert hit['object_key'] == thread_other.object_key


class NoLoginViewTest(CommunityBaseTestCase):
    """
    Test correct url response, without login or security involved
    """

    def test(self):
        response = self.client.get(url_for("forum.index",
                                           community_id=self.community.slug))
        self.assert200(response)


class ViewTestCase(CommunityBaseTestCase):
    no_login = False
    SERVICES = ('security',)

    def test_posts_ordering(self):
        thread = Thread(community=self.community, title=u'test ordering')
        self.session.add(thread)
        t1 = datetime(2014, 6, 20, 15, 0, 0)
        p1 = Post(thread=thread, body_html=u'post 1', created_at=t1)
        t2 = datetime(2014, 6, 20, 15, 1, 0)
        p2 = Post(thread=thread, body_html=u'post 2', created_at=t2)
        self.session.flush()
        p1_id, p2_id = p1.id, p2.id
        assert [p.id for p in thread.posts] == [p1_id, p2_id]

        # set post1 created after post2
        t1 = t1 + timedelta(minutes=2)
        p1.created_at = t1
        self.session.flush()
        self.session.expire(thread)  # force thread.posts refreshed from DB
        assert [p.id for p in thread.posts] == [p2_id, p1_id]

    def test_create_thread_and_post(self):
        # activate email reply
        self.app.config['SBE_FORUM_REPLY_BY_MAIL'] = True
        self.app.config['MAIL_ADDRESS_TAG_CHAR'] = u'+'

        # create a new user, add him/her to the current community as a MANAGER
        self.user = User(email=u'user_1@example.com',
                         password='azerty',
                         can_login=True)
        self.session.add(self.user)
        self.community.set_membership(self.user, MANAGER)
        self.session.commit()
        self.client_login(self.user.email, self.user.password)

        mail = self.app.extensions['mail']
        with mail.record_messages() as outbox:
            title = u"Brand new thread"
            content = u"shiny thread message"
            url = url_for("forum.new_thread", community_id=self.community.slug)
            data = dict(title=title, message=content)
            data['__action'] = u"create"
            data['send_by_email'] = u"y"
            response = self.client.post(url, data=data)
            self.assertStatus(response, 302)

            # extract the thread_id from the redirection url in response
            threadid = response.location.rsplit('/', 2)[1]

            # retrieve the new thread, make sur it has the message
            url = url_for("forum.thread",
                          thread_id=threadid,
                          community_id=self.community.slug,
                          title=title)
            response = self.client.get(url)
            self.assertStatus(response, 200)
            self.assertIn(content, response.data.decode("utf-8"))

            # check the email was sent with the new thread
            assert len(outbox) == 1
            assert outbox[0].subject == u'[My Community] Brand new thread'

        # reset the outbox for checking threadpost email
        with mail.record_messages() as outbox:
            content = data['message'] = u"my cherished post"
            del data['title']
            response = self.client.post(url, data=data)
            self.assertStatus(response, 302)

            # retrieve the new thread, make sur it has the message
            url = url_for("forum.thread",
                          thread_id=threadid,
                          community_id=self.community.slug,
                          title=title)
            response = self.client.get(url)
            self.assertStatus(response, 200)
            self.assertIn(content, response.data.decode("utf-8"))

            # check the email was sent with the new threadpost
            assert len(outbox) == 1
            assert unicode(outbox[
                0].subject) == u'[My Community] Brand new thread'

    def test_create_thread_informative(self):
        """
    Test with 'informative' community. No mail sent, unless user is MANAGER
    """
        assert self.community.type == 'informative'
        # create a new user, add him/her to the current community
        self.user = User(email=u'user_1@example.com',
                         password='azerty',
                         can_login=True)
        self.session.add(self.user)
        self.community.set_membership(self.user, MEMBER)
        self.session.commit()

        title = u"Brand new thread"
        content = u"shiny thread message"
        url = url_for("forum.new_thread", community_id=self.community.slug)
        data = dict(title=title, message=content)
        data['__action'] = u"create"

        mail = self.app.extensions['mail']
        self.client_login(self.user.email, self.user.password)

        with mail.record_messages() as outbox:
            data['send_by_email'] = u"y"  # actually should not be in html form
            response = self.client.post(url, data=data)
            self.assertStatus(response, 302)
            assert len(outbox) == 0

        self.community.set_membership(self.user, MANAGER)
        self.session.commit()

        with mail.record_messages() as outbox:
            data['send_by_email'] = u"y"  # should be in html form
            response = self.client.post(url, data=data)
            self.assertStatus(response, 302)
            assert len(outbox) == 1

        with mail.record_messages() as outbox:
            del data['send_by_email']
            response = self.client.post(url, data=data)
            self.assertStatus(response, 302)
            assert len(outbox) == 0

        self.client_logout()


def get_string_from_file(filename='notification.email'):
    """Load a test email, return as string.
    """
    filepath = Path(__file__).parent / 'data' / filename
    with filepath.open('rt', encoding='utf-8') as email_file:
        email_string = email_file.read()
    return email_string


def get_emailmessage_from_file(filename='notification.email'):
    """Load a mail parse it into a email.message.
    """
    email_string = get_string_from_file(filename)
    parser = FeedParser()
    parser.feed(email_string)
    email_message = parser.close()
    return email_message


class CommandsTest(TestCase):

    @patch('fileinput.input')
    @patch('abilian.sbe.apps.forum.commands.process_email')
    def test_parse_forum_email(self, mock_process_email, mock_email):
        """
        No processing is tested only parsing into a email.message
        and verifying inject_email() logic
        """
        # first load a test email returned by the mock_email
        mock_email.return_value = get_string_from_file('notification.email')

        # test the parsing function
        inject_email()

        # assert the email is read
        assert mock_email.called
        # assert a call on the celery task was made implying a message creation
        assert mock_process_email.delay.called

        ##
        mock_email.reset_mock()
        mock_process_email.delay.reset_mock()
        assert not mock_email.called
        assert not mock_process_email.delay.called

        mock_email.return_value = get_string_from_file('defects.email')
        inject_email()
        assert mock_email.called
        assert not mock_process_email.delay.called


class TasksTest(BaseTestCase):

    def test_build_reply_email_address(self):
        expected_reply_address = 'test+P-fr-3-4-LJXUDQHH3ELLN4KZSLLSA7SHWI@testcase.app.tld'
        self.app.config['MAIL_ADDRESS_TAG_CHAR'] = u'+'
        post = Mock()
        post.id = 2
        post.thread_id = 3
        member = Mock()
        member.id = 4
        with self.app.test_request_context(
                '/build_reply_email_address',
                headers=[('Accept-Language', u'fr')]):
            replyto = build_reply_email_address('test', post, member,
                                                'testcase.app.tld')
        self.assertIn(expected_reply_address, replyto)

    def test_extract_mail_destination(self):
        self.app.config['MAIL_ADDRESS_TAG_CHAR'] = u'+'
        self.app.config['MAIL_SENDER'] = u'test@testcase.app.tld'
        test_address = 'test+test+P-fr-3-4-LJXUDQHH3ELLN4KZSLLSA7SHWI@testcase.app.tld'
        infos = extract_email_destination(test_address)
        self.assertIn(u'fr', infos[0])  # locale
        self.assertIn(u'3', infos[1])  # thread_id
        self.assertIn(u'4', infos[2])  # post.id



from flask import Blueprint

notifications = Blueprint("notifications",
                          __name__,
                          url_prefix="/notifications",
                          template_folder="../templates")



# coding=utf-8
from __future__ import absolute_import

import re
from urllib import quote_plus

from flask import g, url_for
from markdown import Markdown
from mock import MagicMock, patch

from abilian.core.models.subjects import User
from abilian.sbe.apps.communities.tests.base import (CommunityBaseTestCase,
                                                     CommunityIndexingTestCase)
from abilian.sbe.apps.wiki.markup import SBEWikiLinkExtension

from . import views
from .models import WikiPage


def test_wikilink_extension():
    text = u'/*€('
    wikilink = u'[[' + text + ']]'

    def build_url(label, base, end):
        return u'/?title=' + quote_plus(label.encode('utf-8')) + end

    extension = SBEWikiLinkExtension([('build_url', build_url)])
    ctx = {}
    ctx['extensions'] = [extension, 'toc']
    ctx['output_format'] = 'html5'
    md = Markdown(**ctx)

    page_exists_mock = MagicMock(return_value=True)
    with patch('abilian.sbe.apps.wiki.forms.page_exists', page_exists_mock):
        result = md.convert(wikilink)

    qtext = unicode(quote_plus(text.encode('utf-8')))
    expected = u'<p><a class="wikilink" href="/?title={href}/">{text}</a></p>'
    expected = expected.format(href=qtext, text=text)
    assert expected == result

    # make sur the class is 'wikilink new' if page exists'
    page_exists_mock = MagicMock(return_value=False)
    with patch('abilian.sbe.apps.wiki.forms.page_exists', page_exists_mock):
        result = md.convert(wikilink)

    qtext = unicode(quote_plus(text.encode('utf-8')))
    expected = u'<p><a class="wikilink new" href="/?title={href}/">{text}</a></p>'
    expected = expected.format(href=qtext, text=text)
    assert expected == result


class WikiBaseTestCase(CommunityBaseTestCase):

    def setUp(self):
        super(WikiBaseTestCase, self).setUp()
        g.user = User.query.all()[0]


class TestModels(WikiBaseTestCase):

    def test_create_page(self):
        page = WikiPage(title=u"Some page", body_src=u'abc')
        self.assertEqual(page.title, u'Some page')
        self.assertEqual(page.name, u'Some page')
        self.assertEqual(page.body_src, 'abc')
        self.assertEqual(page.body_html, '<p>abc</p>')
        self.assertEqual(len(page.revisions), 1)

        revision = page.revisions[0]
        self.assertEqual(revision.number, 0)
        self.assertEqual(revision.author, g.user)

    def test_rename_page(self):
        page = WikiPage(title=u"Some page", body_src=u'abc')
        self.assertEqual(page.title, u'Some page')
        self.assertEqual(page.name, u'Some page')
        page.title = u'Title Renamed'
        self.assertEqual(page.title, u'Title Renamed')
        self.assertEqual(page.name, u'Title Renamed')

        page.name = u'Name'
        self.assertEqual(page.title, u'Name')
        self.assertEqual(page.name, u'Name')

    def test_create_revision(self):
        page = WikiPage('abc')
        page.create_revision("def", "page updated")

        self.assertEqual(len(page.revisions), 2)
        last_revision = page.revisions[1]
        self.assertEqual(last_revision.number, 1)
        self.assertEqual(last_revision.author, g.user)


class TestIndexing(CommunityIndexingTestCase):

    def setUp(self):
        super(TestIndexing, self).setUp()
        g.user = User.query.all()[0]

    def test_wiki_indexed(self):
        page = WikiPage(title=u'Community 1', community=self.community)
        self.session.add(page)
        page_other = WikiPage(title=u'Community 2: other', community=self.c2)
        self.session.add(page_other)
        self.session.commit()

        svc = self.svc
        obj_types = (WikiPage.entity_type,)
        with self.login(self.user_no_community):
            res = svc.search(u'community', object_types=obj_types)
            self.assertEqual(len(res), 0)

        with self.login(self.user):
            res = svc.search(u'community', object_types=obj_types)
            self.assertEqual(len(res), 1)
            hit = res[0]
            self.assertEqual(hit['object_key'], page.object_key)

        with self.login(self.user_c2):
            res = svc.search(u'community', object_types=obj_types)
            self.assertEqual(len(res), 1)
            hit = res[0]
            self.assertEqual(hit['object_key'], page_other.object_key)


class TestsViews(WikiBaseTestCase):

    def test_home(self):
        response = self.client.get(url_for("wiki.index",
                                           community_id=self.community.slug))
        self.assertStatus(response, 302)

        response = self.client.get(url_for("wiki.page",
                                           title='Home',
                                           community_id=self.community.slug))
        assert response.status_code == 200

    def test_create_page_initial_form(self):
        g.community = self.community
        view = views.PageCreate()
        view.prepare_args([], {})
        form = view.form
        assert form['last_revision_id'].data is None

    def test_create_page(self):
        title = 'Some page name'
        url = url_for("wiki.page_new", community_id=self.community.slug)
        url += '?title=Some+page+name'
        response = self.client.get(url)
        assert response.status_code == 200
        # make sure the title is filled when comming from wikilink
        self.assertIn('value="Some page name"', response.data)

        title = 'Some page'
        body = "LuuP3jai"
        url = url_for("wiki.page_new", community_id=self.community.slug)
        data = dict(title=title, body_src=body, __action="create")
        response = self.client.post(url, data=data)
        assert response.status_code == 302

        url = url_for("wiki.page",
                      community_id=self.community.slug,
                      title=title)
        response = self.client.get(url)
        assert response.status_code == 200
        self.assertIn(body, response.data)

        # edit
        url = url_for("wiki.page_edit",
                      community_id=self.community.slug,
                      title=title)
        response = self.client.get(url)
        assert response.status_code == 200

        # Slightly hackish way to get the page_id
        line = [l for l in response.data.split("\n")
                if 'name="page_id"' in l][0]
        m = re.search('value="(.*?)"', line)
        page_id = int(m.group(1))

        url = url_for("wiki.page_changes",
                      community_id=self.community.slug,
                      title=title)
        response = self.client.get(url)
        assert response.status_code == 200

        url = url_for("wiki.page_source",
                      community_id=self.community.slug,
                      title=title)
        response = self.client.get(url)
        assert response.status_code == 200

        url = url_for("wiki.page_edit", community_id=self.community.slug)
        data = dict(title=title, page_id=page_id, body_src="abc def")
        data['__action'] = 'edit'
        response = self.client.post(url, data=data)
        assert response.status_code == 302

        url = url_for("wiki.page_compare",
                      rev0="on",
                      rev1="on",
                      community_id=self.community.slug,
                      title=title)
        response = self.client.get(url)
        assert response.status_code == 200



# coding=utf-8
"""
"""
from __future__ import absolute_import

import mock
import sqlalchemy as sa
from flask import url_for

from abilian.core.entities import Entity
from abilian.core.models.subjects import User
from abilian.sbe.apps.documents.models import Folder
from abilian.sbe.testing import BaseTestCase
from abilian.services.security import Admin

from .. import signals, views
from ..models import MEMBER, Community, CommunityIdColumn, community_content
from .base import CommunityIndexingTestCase as BaseIndexingTestCase


class CommunityUnitTestCase(BaseTestCase):

    def test_instanciation(self):
        community = Community(name=u"My Community")
        assert isinstance(community.folder, Folder)
        #assert isinstance(community.group, Group)

    def test_default_view_kw(self):
        # test exceptions are handled if passed an object with 'community' attribute
        # and no community_id in kwargs. and ValueError is properly raised
        dummy = type('Dummy', (object,), {'community': None})()
        with self.assertRaises(ValueError) as cm:
            views.default_view_kw({}, dummy, 'dummy', 1)

        assert cm.exception.message == 'Cannot find community_id value'


class CommunityDbTestCase(BaseTestCase):

    def setUp(self):
        super(CommunityDbTestCase, self).setUp()
        self.community = Community(name=u"My Community")
        self.db.session.add(self.community)
        self.db.session.commit()

    def test_default_url(self):
        assert self.app.default_view.url_for(self.community) \
               == '/communities/my-community/'

    def test_can_recreate_with_same_name(self):
        name = self.community.name
        self.db.session.delete(self.community)
        self.db.session.commit()
        self.community = Community(name=name)
        self.db.session.add(self.community)
        # if community.folder was not deleted, this will raise IntegrityError. Test
        # passes if no exceptions is raised
        self.db.session.commit()

    def test_rename(self):
        NEW_NAME = u"My new name"
        self.community.rename(NEW_NAME)
        assert self.community.name == NEW_NAME
        assert self.community.folder.name == NEW_NAME

    def test_auto_slug(self):
        assert self.community.slug == u'my-community'

    def test_membership(self):
        user = User.query.first()
        l = self.community.memberships
        assert l == []

        # setup signals testers with mocks.
        when_set = mock.MagicMock()
        when_set.mock_add_spec(['__name__'])  # required for signals
        signals.membership_set.connect(when_set)
        when_removed = mock.MagicMock()
        when_removed.mock_add_spec(['__name__'])
        signals.membership_removed.connect(when_removed)

        # invalid role
        with self.assertRaises(ValueError):
            self.community.set_membership(user, 'dummy role name')

        assert not when_set.called
        assert not when_removed.called

        # simple member
        self.community.set_membership(user, "member")
        self.db.session.commit()

        l = self.community.memberships
        assert len(l) == 1
        assert l[0].user == user
        assert l[0].role is MEMBER

        when_set.assert_called_once_with(self.community,
                                         is_new=True,
                                         membership=l[0])
        assert not when_removed.called
        when_set.reset_mock()

        assert self.community.get_role(user) is MEMBER

        assert self.community.get_memberships() == [l[0]]
        assert self.community.get_memberships('member') == [l[0]]
        assert self.community.get_memberships('manager') == []

        # change user role
        self.community.set_membership(user, "manager")
        self.db.session.commit()

        l = self.community.memberships
        assert len(l) == 1
        assert l[0].user == user
        assert l[0].role == "manager"

        assert self.community.get_role(user) == "manager"

        when_set.assert_called_once_with(self.community,
                                         is_new=False,
                                         membership=l[0])
        assert not when_removed.called
        when_set.reset_mock()

        # remove user
        membership = l[0]
        self.community.remove_membership(user)
        self.db.session.commit()

        l = self.community.memberships
        assert l == []

        assert not when_set.called
        when_removed.assert_called_once_with(self.community,
                                             membership=membership)

    def test_folder_roles(self):
        user = User.query.first()
        folder = self.community.folder
        self.community.set_membership(user, "member")
        self.db.session.commit()
        security = self.app.services['security']

        assert security.get_roles(user, folder) == ["reader"]

        # this tests a bug, where local roles whould disappear when setting
        # membership twice
        self.community.set_membership(user, "member")
        assert security.get_roles(user, folder) == ["reader"]

    def test_community_content_decorator(self):

        @community_content
        class CommunityContent(Entity):
            community_id = CommunityIdColumn()
            community = sa.orm.relation(Community, foreign_keys=[community_id])

        sa.orm.configure_mappers()
        conn = self.session.connection()
        for table in sa.inspect(CommunityContent).tables:
            if not table.exists(conn):
                table.create(conn)

        cc = CommunityContent(name=u'my content', community=self.community)
        self.session.add(cc)
        self.session.flush()
        assert hasattr(cc, 'community_slug')
        assert cc.community_slug == 'my-community'
        assert cc.slug == u'my-content'

        index_to = dict(CommunityContent.__indexation_args__['index_to'])
        assert 'community_slug' in index_to


class CommunityIndexingTestCase(BaseIndexingTestCase):

    def test_community_indexed(self):
        svc = self.svc
        obj_types = (Community.entity_type,)
        with self.login(self.user_no_community):
            res = svc.search(u'community', object_types=obj_types)
            assert len(res) == 0

        with self.login(self.user):
            res = svc.search(u'community', object_types=obj_types)
            assert len(res) == 1
            hit = res[0]
            assert hit['object_key'] == self.community.object_key

        with self.login(self.user_c2):
            res = svc.search(u'community', object_types=obj_types)
            assert len(res) == 1
            hit = res[0]
            assert hit['object_key'] == self.c2.object_key

    def test_default_view_kw_with_hit(self):
        with self.login(self.user):
            hit = self.svc.search(u'community',
                                  object_types=(Community.entity_type,))[0]
            kw = views.default_view_kw({}, hit, hit['object_type'], hit['id'])

        assert kw == {'community_id': self.community.slug}


class CommunityWebTestCase(BaseIndexingTestCase):

    # FIXME later
    SQLALCHEMY_WARNINGS_AS_ERROR = False

    def test_index(self):
        with self.client_login(self.user.email, 'azerty'):
            response = self.client.get(url_for("communities.index"))
        self.assert_200(response)

    def test_community_home(self):
        url = self.app.default_view.url_for(self.community)
        user = self.user.email
        user_c2 = self.user_c2.email
        with self.client_login(user_c2, 'azerty'):
            response = self.client.get(url)
            assert response.status_code == 403

        with self.client_login(user, 'azerty'):
            response = self.client.get(url)
            assert response.status_code == 302
            expected_url = url_for("wall.index",
                                   community_id=self.community.slug,
                                   _external=True)
            assert response.headers['Location'] == expected_url

    def test_community_settings(self):
        url = url_for('communities.settings', community_id=self.community.slug)
        with self.client_login(self.user.email, 'azerty'):
            response = self.client.get(url)
            assert response.status_code == 403

            self.app.services['security'].grant_role(self.user, Admin)
            response = self.client.get(url)
            self.assert_200(response)

            data = {'__action': u'edit',
                    'name': u'edited community',
                    'description': u'my community',
                    'linked_group': u'',
                    'type': 'participative',}
            response = self.client.post(url, data=data)
            assert response.status_code == 302
            self.assertEqual(
                response.headers['Location'],
                u'http://localhost/communities/{}/'.format(self.community.slug))

            community = Community.query.get(self.community.id)
            assert community.name == u'edited community'

    def test_new(self):
        with self.client_login(self.user.email, 'azerty'):
            response = self.client.get(url_for("communities.new"))
            assert response.status_code == 403

            self.app.services['security'].grant_role(self.user, Admin)
            response = self.client.get(url_for("communities.new"))
            assert response.status_code == 200

    def test_members(self):
        with self.client_login(self.user.email, 'azerty'):
            url = url_for("communities.members",
                          community_id=self.community.slug)
            response = self.client.get(url)
            self.assert_200(response)

            # test add user
            data = {'action': 'add-user-role', 'user': self.user_c2.id}
            response = self.client.post(url, data=data)
            assert response.status_code == 403

            self.app.services['security'].grant_role(self.user, Admin)
            data = {
                'action': 'add-user-role',
                'user': self.user_c2.id,
                'role': 'member',
            }
            response = self.client.post(url, data=data)
            assert response.status_code == 302
            response.headers['Location'] == 'http://localhost' + url

            membership = [m for m in self.community.memberships
                          if m.user == self.user_c2][0]
            assert membership.role == 'member'

            data['action'] = 'set-user-role'
            data['role'] = 'manager'
            response = self.client.post(url, data=data)
            assert response.status_code == 302
            assert response.headers['Location'] == 'http://localhost' + url
            self.session.expire(membership)
            assert membership.role == 'manager'

            # Community.query.session is not self.db.session, but web app session.
            community = Community.query.get(self.community.id)
            assert self.user_c2 in community.members

            # test delete
            data = {
                'action': 'delete',
                'user': self.user_c2.id,
                'membership': [m.id for m in community.memberships
                               if m.user == self.user_c2][0],
            }
            response = self.client.post(url, data=data)
            assert response.status_code == 302
            self.assertEqual(response.headers['Location'],
                             u'http://localhost/communities/{}/members'.format(
                                 self.community.slug))

            assert self.user_c2 not in community.members



# coding=utf-8
"""
Forum views
"""
from __future__ import absolute_import, print_function, unicode_literals

from datetime import date, datetime
from itertools import groupby
from urllib import quote

import sqlalchemy as sa
from flask import (current_app, flash, g, make_response, render_template,
                   request)
from flask_babel import format_date
from flask_login import current_user
from sqlalchemy.orm import joinedload
from werkzeug.exceptions import BadRequest, NotFound

from abilian.core.util import utc_dt
from abilian.i18n import _, _l
from abilian.sbe.apps.calendar.forms import EventForm
from abilian.sbe.apps.calendar.models import Event
from abilian.sbe.apps.communities.blueprint import Blueprint
from abilian.sbe.apps.communities.views import default_view_kw
from abilian.web import url_for, views
from abilian.web.action import ButtonAction, Endpoint
from abilian.web.nav import BreadcrumbItem
from abilian.web.views import default_view

blueprint = Blueprint("calendar",
                      __name__,
                      url_prefix="/calendar",
                      template_folder="templates")
route = blueprint.route


@route('/')
def index():
    return render_template('calendar/index.html')


class BaseEventView(object):
    Model = Event
    Form = EventForm
    pk = 'event_id'
    base_template = 'community/_base.html'

    def index_url(self):
        return url_for(".index", community_id=g.community.slug)

    def view_url(self):
        return url_for(self.obj)


class EventCreateView(BaseEventView, views.ObjectCreate):
    POST_BUTTON = ButtonAction('form',
                               'create',
                               btn_class='primary',
                               title=_l('Post this event'))

    title = _("New event")

    def init_object(self, args, kwargs):
        args, kwargs = super(EventCreateView, self).init_object(args, kwargs)
        self.event = self.obj
        return args, kwargs

    @property
    def activity_target(self):
        return self.event.community

    def get_form_buttons(self, *args, **kwargs):
        return [self.POST_BUTTON, views.object.CANCEL_BUTTON]


route('/new_event/')(EventCreateView.as_view(b'new_event',
                                             view_endpoint='.event'))



#!./bin/python

import subprocess
import urllib
import time

# Some random number
BIND = '0.0.0.0'
PORT = 4034
HOME = "http://{}:{}/".format(BIND, PORT)

subprocess.call(['./bin/pip', 'install', 'gunicorn'])

p = subprocess.Popen(["./bin/gunicorn", 'wsgi:app', '-b', '{}:{}'.format(BIND,
                                                                         PORT)])

try:
    # Just in case
    time.sleep(5)
    page = urllib.urlopen(HOME).read()
    assert "Welcome to Abilian" in page
finally:
    p.terminate()
    p.wait()



# coding=utf-8
from flask import url_for
from flask_babel import lazy_gettext as _l

from abilian.sbe.apps.communities.actions import CommunityEndpoint
from abilian.web.action import Action, FAIcon, ModalActionMixin, actions


class WikiPageAction(Action):
    Endpoint = CommunityEndpoint

    def pre_condition(self, context):
        page = context.get('object')
        return bool(page)

    def url(self, context=None):
        if self._url:
            return self._url
        else:
            page = context.get('object')
            kw = self.endpoint.get_kwargs()
            kw['title'] = page.title
            return url_for(self.endpoint.name, **kw)


class WikiPageModalAction(ModalActionMixin, WikiPageAction):
    pass


class WikiAction(Action):
    Endpoint = CommunityEndpoint


_actions = (WikiPageAction('wiki:page',
                           'view',
                           _l(u'View'),
                           endpoint='.page',
                           icon='eye-open'),
            WikiPageAction('wiki:page',
                           'edit',
                           _l(u'Edit'),
                           endpoint='.page_edit',
                           icon='pencil'),
            WikiPageModalAction('wiki:page',
                                'upload_attachment',
                                _l(u'Upload an attachment'),
                                url='#upload-files',
                                icon='plus'),
            WikiPageAction('wiki:page',
                           'source',
                           _l(u'Source'),
                           endpoint='.page_source',
                           icon=FAIcon('code')),
            WikiPageAction('wiki:page',
                           'changes',
                           _l(u'Changes'),
                           endpoint='.page_changes',
                           icon='time'),
            WikiPageModalAction('wiki:page',
                                'delete',
                                _l(u'Delete'),
                                url='#modal-delete',
                                icon='trash'),
            WikiAction('wiki:global',
                       'new',
                       _l(u'New page'),
                       endpoint='.page_new',
                       icon='plus'),
            WikiAction('wiki:global',
                       'pages',
                       _l(u'All pages'),
                       endpoint='.wiki_pages',
                       icon='list'),
            WikiAction('wiki:global',
                       'help',
                       _l(u'Syntax help'),
                       endpoint='.wiki_help',
                       icon='info-sign'),)


def register_actions(state):
    if not actions.installed(state.app):
        return

    with state.app.app_context():
        actions.register(*_actions)



from __future__ import print_function

import os.path
import uuid

from flask import Blueprint, current_app, request
from flask.ext.login import current_user
from lxml.etree import XMLSyntaxError
from werkzeug.datastructures import Headers
from werkzeug.exceptions import Forbidden, NotFound
from werkzeug.wrappers import BaseResponse as Response

from abilian.core.extensions import db

from .. import repository
from .constants import (DAV_PROPS, HTTP_BAD_REQUEST, HTTP_CONFLICT,
                        HTTP_CREATED, HTTP_METHOD_NOT_ALLOWED,
                        HTTP_MULTI_STATUS, HTTP_NO_CONTENT, HTTP_OK,
                        HTTP_PRECONDITION_FAILED, OPTIONS)
from .xml import MultiStatus, Propfind

webdav = Blueprint("webdav", __name__, url_prefix="/webdav")
route = webdav.route

__all__ = ['webdav']

#
# Utils
#


# TODO: real logging
class Logger(object):

    def debug(self, msg):
        print(msg)


log = Logger()


# XXX: temporary debug info.
@webdav.before_request
def log_request():
    litmus_msg = request.headers.get('X-Litmus')
    if litmus_msg:
        print('')
        print(78 * '#')
        print(litmus_msg)
        print('{} on {}'.format(request.method, request.path))


@webdav.before_request
def only_admin():
    if not current_app.services('security').has_role(current_user, "admin"):
        raise Forbidden()


@webdav.after_request
def log_response(response):
    print('Response: {}'.format(response))
    return response


def normpath(path):
    path = os.path.normpath(path)
    if not path.startswith("/"):
        path = "/" + path
    return path


def split_path(path):
    path = normpath(path)
    return os.path.dirname(path), os.path.basename(path)


def get_object(path):
    obj = repository.get_object_by_path(path)
    if obj is None:
        raise NotFound()
    return obj


@webdav.before_app_request
def create_root_folder():
    # TODO: create root folder on repository startup instead.
    #assert repository.root_folder
    pass


#
# HTTP endpoints
#
@route('/', methods=['OPTIONS'], defaults={'path': '/'})
@route('/<path:path>', methods=['OPTIONS'])
def options(path):
    headers = {'Content-Type': 'text/plain',
               'Content-Length': '0',
               'DAV': '1,2',
               'MS-Author-Via': 'DAV',
               'Allow': OPTIONS}
    print("Returning", headers)
    return "", HTTP_OK, headers


@route('/', defaults={'path': '/'})
@route('/<path:path>')
def get(path):
    path = normpath(path)

    obj = get_object(path)

    file_name = obj.file_name.encode('utf8')
    headers = {'Content-Type': obj.content_type,
               'Content-Disposition': 'attachment;filename=%s' % file_name}
    return obj.content, HTTP_OK, headers


@route('/<path:path>', methods=['MKCOL'])
def mkcol(path):
    path = normpath(path)
    parent_path, name = split_path(path)

    if request.data:
        return "Request body must be empty.", 415, {}

    obj = repository.get_object_by_path(path)
    if obj is not None:
        return "Objet already exists.", HTTP_METHOD_NOT_ALLOWED, {}

    parent_folder = repository.get_folder_by_path(parent_path)
    if parent_folder is None:
        return "Parent collection doesn't exist.", HTTP_CONFLICT, {}

    new_folder = parent_folder.create_subfolder(name=name)  # noqa
    db.session.commit()
    return "", HTTP_CREATED, {}


@route('/<path:path>', methods=['PUT'])
def put(path):
    path = normpath(path)
    parent_path, name = split_path(path)

    status = HTTP_CREATED
    obj = repository.get_object_by_path(path)
    if obj is not None:
        if not obj.is_document:
            return "", HTTP_METHOD_NOT_ALLOWED, {}
        else:
            status = HTTP_NO_CONTENT
    else:
        parent_folder = repository.get_folder_by_path(parent_path)
        if parent_folder is None:
            return "", HTTP_CONFLICT, {}
        obj = parent_folder.create_document(name=name)

    obj.content = request.data
    obj.content_type = request.content_type

    db.session.commit()
    return "", status, {}


@route('/<path:path>', methods=['DELETE'])
def delete(path):
    path = normpath(path)

    obj = get_object(path)

    db.session.delete(obj)
    db.session.commit()
    return "", HTTP_NO_CONTENT, {}


@route("/<path:path>", methods=['COPY', 'MOVE'])
def copy_or_move(path):
    path = normpath(path)
    dest = request.headers.get('destination')
    dest_path = normpath(dest[len(request.url_root + "webdav"):])
    dest_parent_path, dest_name = split_path(dest_path)
    overwrite = request.headers.get('overwrite')

    orig_obj = get_object(path)

    status = HTTP_CREATED
    dest_obj = repository.get_object_by_path(dest_path)
    if dest_obj:
        if overwrite == "F":
            return "", HTTP_PRECONDITION_FAILED, {}
        else:
            repository.delete_object(dest_obj)
            db.session.flush()
            status = HTTP_NO_CONTENT

    dest_folder = repository.get_folder_by_path(dest_parent_path)
    if dest_folder is None:
        return "", HTTP_CONFLICT, {}

    if request.method == 'COPY':
        repository.copy_object(orig_obj, dest_folder, dest_name)
    else:
        if dest_folder == orig_obj.parent:
            repository.rename_object(orig_obj, dest_name)
        else:
            repository.move_object(orig_obj, dest_folder, dest_name)

    db.session.commit()
    return "", status, {}


@route('/', defaults={'path': '/'}, methods=['PROPFIND'])
@route('/<path:path>', methods=['PROPFIND'])
def propfind(path):
    path = normpath(path)
    depth = request.headers.get('depth', "1")

    print(request.headers)
    print(request.data)

    try:
        propfind = Propfind(request.data)  # noqa
    except XMLSyntaxError:
        return "Malformed XML document.", HTTP_BAD_REQUEST, {}

    obj = get_object(path)

    m = MultiStatus()
    m.add_response_for(request.url, obj, DAV_PROPS)

    if depth == "1" and obj.is_folder:
        for child in obj.children:
            m.add_response_for(request.url + "/" + child.name, child, DAV_PROPS)

    print(m.to_string())

    headers = {'Content-Type': 'text/xml'}
    return m.to_string(), HTTP_MULTI_STATUS, headers


@route('/<path:path>', methods=['LOCK'])
def lock(path):
    path = normpath(path)
    obj = get_object(path)
    token = str(uuid.uuid1())

    if repository.is_locked(obj):
        if not repository.can_unlock(obj):
            return "", 423, {}
        else:
            headers = {"Lock-Token": "urn:uuid:" + token}
            return "TODO", HTTP_OK, headers

    token = repository.lock(obj)

    xml = '''<?xml version="1.0" encoding="utf-8" ?>
<D:prop xmlns:D="DAV:">
    <D:lockdiscovery>
        <D:activelock>
            <D:lockscope><D:exclusive/></D:lockscope>
            <D:locktype><D:write/></D:locktype>
            <D:depth>0</D:depth>
            <D:timeout>Second-179</D:timeout>
            <D:owner>flora</D:owner>
            <D:locktoken>
                <D:href>opaquelocktoken:%s</D:href>
            </D:locktoken>
        </D:activelock>
    </D:lockdiscovery>
</D:prop>''' % token

    hlist = [('Content-Type', 'text/xml'),
             ('Lock-Token', '<urn:uuid:%s>' % token)]

    return Response(xml,
                    headers=Headers.linked(hlist))  # , status ='423 Locked'
    """
    public Response lock(@Context UriInfo uriInfo) throws Exception {
        String token = null;
        Prop prop = null;
        if (backend.isLocked(doc.getRef())) {
            if (!backend.canUnlock(doc.getRef())) {
                return Response.status(423).build();
            } else {
                token = backend.getCheckoutUser(doc.getRef());
                prop = new Prop(getLockDiscovery(doc, uriInfo));
                return Response.ok().entity(prop).header("Lock-Token",
                        "urn:uuid:" + token).build();
            }
        }

        token = backend.lock(doc.getRef());
        if (READONLY_TOKEN.equals(token)) {
            return Response.status(423).build();
        } else if (StringUtils.isEmpty(token)) {
            return Response.status(400).build();
        }

        prop = new Prop(getLockDiscovery(doc, uriInfo));

        backend.saveChanges();
        return Response.ok().entity(prop).header("Lock-Token",
                "urn:uuid:" + token).build();
    }

  """


@route('/<path:path>', methods=['UNLOCK'])
def unlock(path):
    path = normpath(path)
    obj = get_object(path)

    if repository.is_locked(obj):
        if not repository.can_unlock(obj):
            return "", 423, {}
        else:
            repository.unlock(obj)
            db.session.commit()
            return "", HTTP_NO_CONTENT, {}

    return "", HTTP_NO_CONTENT, {}
    """
        if (backend.isLocked(doc.getRef())) {
            if (!backend.canUnlock(doc.getRef())) {
                return Response.status(423).build();
            } else {
                backend.unlock(doc.getRef());
                backend.saveChanges();
                return Response.status(HTTP_NO_CONTENT).build();
            }
        } else {
            // TODO: return an error
            return Response.status(HTTP_NO_CONTENT).build();
        }


  """



# coding=utf-8

from flask import current_app, g, url_for
from flask_login import current_user

from abilian.i18n import _l
from abilian.services.security import Admin
from abilian.web.action import Action, FAIcon, ModalActionMixin, actions


class ForumAction(Action):

    def url(self, context=None):
        if self._url or self.endpoint:
            return super(ForumAction, self).url(context=context)

        return url_for("." + self.name, community_id=g.community.slug)


class ThreadAction(ForumAction):

    def pre_condition(self, context):
        thread = actions.context.get('object')
        return not not thread


def is_admin(context):
    svc = current_app.services['security']
    return svc.has_role(current_user, Admin, object=context.get('object'))


def is_in_thread(context):
    thread = context.get('object')
    return not thread


def is_closed(context):
    thread = context.get('object')
    return thread.closed


def not_closed(context):
    return not is_closed(context)


class ForumModalAction(ModalActionMixin, ThreadAction):
    pass


_close_template_action = u'''
<form method="POST" action="{{ url }}" encoding="multipart/form-data">
  {{ csrf.field() }}
  <button type="submit" class="btn btn-link" name="action"
          value="{{ action.name}}">
    {%- if action.icon %}{{ action.icon }} {% endif %}
    {{ action.title }}
  </button>
</form>
'''

_actions = (
    ForumAction('forum:global',
                'new_thread',
                _l(u'Start a new conversation'),
                icon='plus'),
    ForumAction('forum:global',
                'index',
                _l(u'Recent conversations'),
                icon='list'),
    ForumAction('forum:global',
                'archives',
                _l(u'Archives'),
                icon='calendar'),
    ForumAction('forum:global',
                'attachments',
                _l(u'Attachments'),
                icon='file',
                condition=is_in_thread),
    ForumModalAction('forum:thread',
                     'delete',
                     _l(u'Delete'),
                     condition=lambda ctx: is_admin(ctx) and not_closed(ctx),
                     url='#modal-delete',
                     icon='trash'),
    ThreadAction('forum:thread',
                 'close',
                 _l('Close thread'),
                 url='close',
                 template_string=_close_template_action,
                 condition=lambda ctx: is_admin(ctx) and not_closed(ctx),
                 icon=FAIcon('lock')),
    ThreadAction('forum:thread',
                 'reopen',
                 _l('Re-open thread'),
                 url='close',
                 template_string=_close_template_action,
                 condition=lambda ctx: is_admin(ctx) and is_closed(ctx),
                 icon=FAIcon('unlock')),
    ThreadAction('forum:thread',
                 'attachments',
                 _l(u'Attachments'),
                 url='attachments',
                 icon='file'),)


def register_actions(state):
    if not actions.installed(state.app):
        return
    with state.app.app_context():
        actions.register(*_actions)



# coding=utf-8
"""
"""
from __future__ import absolute_import

from datetime import datetime, timedelta

import dateutil.parser
from flask import current_app
from flask_login import current_user
from future.utils import raise_from

from abilian.core.util import utcnow

DEFAULT_LIFETIME = 3600


class Lock(object):
    """
    Represent a lock on a document
    """

    def __init__(self, user_id, user, date, *args, **kwargs):
        self.user_id = user_id
        self.user = user
        if not isinstance(date, datetime):
            try:
                date = dateutil.parser.parse(date)
            except Exception as e:
                raise_from(
                    ValueError('Error parsing date: {!r}'.format(date)), e)

        self.date = date

    @staticmethod
    def new():
        return Lock(current_user.id, unicode(current_user), utcnow())

    def as_dict(self):
        """
        return a dict suitable for serialization to JSON
        """
        return dict(user_id=self.user_id,
                    user=self.user,
                    date=self.date.isoformat())

    @staticmethod
    def from_dict(d):
        """
        Deserialize from a `dict` created by :meth:`as_dict`.
        """
        return Lock(**d)

    @property
    def lifetime(self):
        return current_app.config.get('SBE_LOCK_LIFETIME', DEFAULT_LIFETIME)

    @property
    def expired(self):
        return (utcnow() - self.date) > timedelta(seconds=self.lifetime)

    def is_owner(self, user=None):
        if user is None:
            user = current_user

        return self.user_id == user.id



# coding=utf-8
"""
"""
from __future__ import absolute_import

from datetime import datetime, timedelta

import mock
from pytz import UTC

from abilian.core.models.subjects import User
from abilian.sbe.apps.documents import lock
from abilian.sbe.apps.documents.lock import Lock
from abilian.testing import BaseTestCase


def test_lock():
    date = datetime(2015, 10, 22, 14, 58, 42, tzinfo=UTC)
    l = Lock(user_id=3, user=u'Joe Smith', date=date)

    d = l.as_dict()
    assert d == dict(user_id=3,
                     user=u'Joe Smith',
                     date=u'2015-10-22T14:58:42+00:00')
    l = Lock.from_dict(d)
    assert l.user_id == 3
    assert l.user == u'Joe Smith'
    assert l.date == date


class LockTestCase(BaseTestCase):

    SERVICES = ('security',)

    def test_lock(self):
        session = self.app.db.session()
        user = User(email=u'test@example.com',
                    first_name=u'Joe',
                    last_name=u'Smith',
                    can_login=True)
        other = User(email=u'other@exemple.com')
        session.add(user)
        session.add(other)
        session.commit()

        # set 30s lifetime
        self.app.config['SBE_LOCK_LIFETIME'] = 30
        dt_patcher = mock.patch.object(
            lock, 'utcnow', mock.Mock(wraps=lock.utcnow))
        with dt_patcher as mocked:
            created_at = datetime(2015, 10, 22, 14, 58, 42, tzinfo=UTC)
            mocked.return_value = created_at

            self.login(user)
            l = Lock.new()
            assert l.user_id == user.id
            assert l.user == u'Joe Smith'
            assert l.date == created_at
            assert l.is_owner()
            assert l.is_owner(user)
            assert not l.is_owner(other)

            assert l.lifetime == 30
            mocked.return_value = created_at + timedelta(seconds=40)
            assert l.expired

            mocked.return_value = created_at + timedelta(seconds=20)
            assert not l.expired

            self.login(other)
            assert not l.is_owner()



# coding=utf-8
from os.path import dirname, join

from flask import (current_app, flash, g, jsonify, make_response, redirect,
                   render_template, request, url_for)
from flask_babel import gettext as _
from sqlalchemy import func
from werkzeug.exceptions import InternalServerError, NotFound

from abilian.core.extensions import db
from abilian.core.models.subjects import Group, User
from abilian.services.image import CROP, resize
from abilian.services.security import security
from abilian.web import csrf
from abilian.web.views import default_view

from .social import social
from .util import Env

DEFAULT_GROUP_MUGSHOT = open(join(
    dirname(__file__), "../../../static/images/frog.jpg")).read()


@social.route("/groups/")
def groups():
    tab = request.args.get("tab", "all_groups")
    e = Env()
    if tab == 'all_groups':
        e.groups = Group.query.order_by(Group.name).all()
        if not security.has_role(g.user, "admin"):
            e.groups = [group for group in e.groups
                        if group.public or g.user in group.members]
    else:
        e.groups = g.user.groups
        e.groups.sort(key=lambda x: x.name)
    return render_template("social/groups.html", **e)


def is_admin(group):
    is_admin = g.user in group.admins
    if not is_admin and 'security' in current_app.extensions:
        is_admin = current_app.services['security'].has_role(g.user, 'admin')

    return is_admin


@social.route("/groups/<int:group_id>")
@default_view(social, Group, 'group_id')
def group_home(group_id):
    e = Env(csrf_token=csrf.field())
    e.group = Group.query.get(group_id)
    e.is_admin = is_admin(e.group)
    return render_template("social/group.html", **e)


@social.route("/groups/<int:group_id>/json")
def group_json(group_id):
    members = Group.query.get(group_id).members
    q = request.args.get("q", u'').lower()
    if q:
        members = filter(
            lambda u: any(term.startswith(q)
                          for name in (u.first_name.lower(), u.last_name.lower())
                          for term in name.split()),
            members)

    result = {'results': [{'id': obj.id, 'text': obj.name} for obj in members]}
    return jsonify(result)


@social.route("/groups/<int:group_id>", methods=['POST'])
@csrf.protect
def group_post(group_id):
    group = Group.query.get(group_id)
    action = request.form.get('action')
    return_url = request.form.get('return_url')

    membership_actions = frozenset(('add',
                                    'remove',
                                    'add-admin',
                                    'remove-admin',))

    if action not in ('join', 'leave'):
        assert is_admin(group)

    if action == 'join':
        g.user.join(group)
    elif action == 'leave':
        g.user.leave(group)
    elif action in membership_actions:
        user_id = request.form.get('user', u'').strip()
        try:
            user_id = int(user_id)
        except:
            flash(_(u'Error: No user selected'), 'error')
            return redirect(url_for(".group_home", group_id=group_id))

        user = User.query.get(user_id)

        if action == 'add':
            user.join(group)
        elif action == 'remove':
            user.leave(group)
        elif action == 'add-admin':
            if user not in group.admins:
                group.admins.append(user)
        elif action == 'remove-admin':
            if user in group.admins:
                group.admins.remove(user)
    else:
        raise Exception("Should not happen")
    db.session.commit()

    if return_url:
        # TODO: security check
        return redirect(return_url)
    else:
        return redirect(url_for(".group_home", group_id=group_id))


@social.route("/groups/new")
def groups_new():
    # TODO later
    return

    # e = Env()
    # e.form = GroupForm()
    # return render_template("social/groups-new.html", **e)


@social.route("/groups/new", methods=['POST'])
def groups_new_post():
    # TODO later
    return

    # form = GroupForm()
    #
    # if form.validate():
    #     group = Group()
    #     form.populate_obj(group)
    #     db.session.add(group)
    #     db.session.commit()
    #     flash(_(u"Your new group has been created"), category='info')
    #     return redirect(url_for('.group_home', group_id=group.id))
    #
    # else:
    #     e = Env()
    #     e.form = form
    #     return render_template("social/groups-new.html", **e)


@social.route("/groups/<int:group_id>/mugshot")
def group_mugshot(group_id):
    # TODO: duplicated code (with user_mugshot). Extract common method.
    size = int(request.args.get('s', 55))
    if size > 500:
        raise ValueError("Error, size = %d" % size)
    group = Group.query.get(group_id)

    if not group:
        raise NotFound()

    if group.photo:
        data = group.photo
    else:
        data = DEFAULT_GROUP_MUGSHOT

    if size:
        data = resize(data, size, size, mode=CROP)

    response = make_response(data)
    response.headers['content-type'] = 'image/jpeg'
    response.headers.add('Cache-Control', 'public, max-age=600')
    return response


#
# Ad-hoc JSON endpoints, used by select boxes
#
@social.route("/groups/json")
def groups_json():
    q = request.args.get("q").replace("%", " ").lower()

    if not q or len(q) < 2:
        raise InternalServerError()

    query = Group.query
    #query = query.filter(func.lower(Group.name).like(q + "%"))
    query = query.order_by(func.lower(Group.name))
    all = query.all()

    result = {'results': [{'id': obj.id, 'text': obj.name} for obj in all]}
    return jsonify(result)



# coding=utf-8
"""
"""
from __future__ import absolute_import

import setuptools
from distutils.command.build import build as _build
from setuptools.command.sdist import sdist as _sdist
from setuptools.command.develop import develop as _develop

import pip

session = pip.download.PipSession()

_install_requires = pip.req.parse_requirements(
    'requirements.in', session=session)
install_requires = [str(ir.req) for ir in _install_requires]

_dev_requires = pip.req.parse_requirements(
    'etc/dev-requirements.txt', session=session)
dev_requires = [str(ir.req) for ir in _dev_requires]

LONG_DESCRIPTION = open('README.rst', 'r').read()


class build(_build):
  sub_commands = [('compile_catalog', None)] + _build.sub_commands


class sdist(_sdist):
  sub_commands = [('compile_catalog', None)] + _sdist.sub_commands


class develop(_develop):
  def run(self):
    _develop.run(self)
    self.run_command('compile_catalog')


setuptools.setup(
  name='abilian-sbe',
  use_scm_version=True,
  url='https://github.com/abilian/abilian-sbe',
  license='LGPL',
  author='Abilian SAS',
  author_email='contact@abilian.com',
  description='Social Business platform, including: document management, wiki, forum, enterprise social networking, and more',
  long_description=LONG_DESCRIPTION,
  packages=['abilian.sbe'],
  zip_safe=False,
  platforms='any',
  setup_requires=['babel', 'setuptools-git', 'setuptools_scm>=1.5.5'],
  install_requires=install_requires,
  extras_require={
    'tests': dev_requires,
    'dev': dev_requires,
  },
  # dependency_links=dependency_links,
  include_package_data=True,
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Environment :: Web Environment',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: GNU Lesser General Public License v2 (LGPLv2)',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Framework :: Flask',
  ],
  cmdclass={
    'build': build,
    'sdist': sdist,
    'develop': develop,
  },
)
